package com.fjhx.xmhjc.controller.open;


import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.xmhjc.anno.LoginValid;
import com.fjhx.xmhjc.entity.product.vo.ProductCategoryVo;
import com.fjhx.xmhjc.entity.topic.dto.TopicContentDto;
import com.fjhx.xmhjc.entity.topic.dto.TopicContentSelectDto;
import com.fjhx.xmhjc.entity.topic.dto.TopicRepliesDto;
import com.fjhx.xmhjc.entity.topic.dto.TopicRepliesSelectDto;
import com.fjhx.xmhjc.entity.topic.po.TopicContent;
import com.fjhx.xmhjc.entity.topic.po.TopicReplies;
import com.fjhx.xmhjc.entity.topic.vo.OpenTopicContentVO;
import com.fjhx.xmhjc.entity.topic.vo.TopicContentVo;
import com.fjhx.xmhjc.entity.topic.vo.TopicRepliesVo;
import com.fjhx.xmhjc.entity.website.po.WebsiteUsers;
import com.fjhx.xmhjc.service.topic.TopicContentService;
import com.fjhx.xmhjc.service.topic.TopicRepliesService;
import com.fjhx.xmhjc.service.website.WebsiteUsersService;
import com.fjhx.xmhjc.utils.WebsiteUserUtil;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/open/topic")
public class OpenTopicController {
    @Resource
    private TopicContentService topicContentService;
    @Resource
    private TopicRepliesService topicRepliesService;
    @Resource
    private WebsiteUsersService websiteUsersService;

    /**
     * @description: 主题列表
     * @author hj
     * @date 2023/11/23 22:29
     */
    @PostMapping("/contentPage/{sortType}")
    public Page<TopicContentVo> contentPage(@RequestBody TopicContentSelectDto dto, @PathVariable String sortType) {
        dto.setType("1");
        Page<TopicContentVo> page = topicContentService.contentPage(dto, sortType);
        page.getRecords().forEach(topicContentVo -> {
            topicContentVo.setAuthorName(websiteUsersService.getAuthorName(topicContentVo.getAuthor()));
            //获取置顶回复，或者是最新回复
            TopicReplies topReplies = topicRepliesService.getTopReplies(topicContentVo.getId());
            if(ObjectUtil.isNotNull(topReplies) && topReplies.getId() != null){
                topicContentVo.setOutline(topReplies.getContent());
            }else{
                TopicReplies newReplies = topicRepliesService.getNewReplies(topicContentVo.getId());
                if(ObjectUtil.isNotNull(newReplies) && newReplies.getId() != null){
                    topicContentVo.setOutline(newReplies.getContent());
                }
            }
        });
        return page;
    }


    /**
     * 回复列表
     * @author hj
     * @date 2023/11/23 22:29
     */
    @PostMapping("/replyPage")
    public Page<TopicRepliesVo> replyPage(@RequestBody TopicRepliesSelectDto dto) {
        if(!NumberUtil.isValidNumber(dto.getFloorId())){
            throw new RuntimeException("参数错误");
        }
        Map<Long, WebsiteUsers> websiteUsersMap = new HashMap<>();
        Page<TopicRepliesVo> page = topicRepliesService.replyPage(dto);
        page.getRecords().forEach(topicContentVo -> {
            topicContentVo.setAuthorName(websiteUsersService.getAuthorName(topicContentVo.getAuthor(), websiteUsersMap));

            WebsiteUsers websiteUsers = websiteUsersMap.get(topicContentVo.getAuthor());
            if(ObjectUtil.isNull(websiteUsers)){
                websiteUsers = websiteUsersService.getById(topicContentVo.getAuthor());
                websiteUsersMap.put(topicContentVo.getAuthor(),websiteUsers);
            }
            topicContentVo.setCiteAuthorName(websiteUsersService.getAuthorName(topicContentVo.getCiteAuthor(), websiteUsersMap));
        });
        return page;
    }

    /**
     * 创建提问
     * @author hj
     * @date 2023/11/23 22:29
     */
    @LoginValid
    @PostMapping("/createQuestion")
    public void createQuestion(@RequestBody TopicContentDto topicContentDto) {
        if (StrUtil.isBlank(topicContentDto.getTitle())) {
            throw new RuntimeException("标题不能为空");
        }
        if (StrUtil.isBlank(topicContentDto.getContent())) {
            throw new RuntimeException("内容不能为空");
        }
        topicContentDto.setType("1");
        WebsiteUsers loginWebsiteUser = WebsiteUserUtil.getLoginWebsiteUser();
        topicContentDto.setAuthor(loginWebsiteUser.getId());
        topicContentService.add(topicContentDto);
    }


    /**
     * 回复主题/楼层
     * @author hj
     * @date 2023/11/23 22:30
     */
    @LoginValid
    @PostMapping("/reply/{topicId}")
    public void reply(@RequestBody TopicRepliesDto topicRepliesDto, @PathVariable Long topicId) {
        validateReply(topicRepliesDto, topicId);
        WebsiteUsers loginWebsiteUser = WebsiteUserUtil.getLoginWebsiteUser();
        topicRepliesDto.setAuthor(loginWebsiteUser.getId());
        topicRepliesService.add(topicRepliesDto);
    }

    /**
     * 主题详情
     * @author hj
     * @date 2023/11/23 22:30
     */
    @PostMapping("/detail/{topicId}")
    public OpenTopicContentVO detail(@PathVariable Long topicId,@RequestBody TopicRepliesSelectDto dto) {
        TopicContent topicContent = topicContentService.getById(topicId);
        if (ObjectUtil.isNull(topicContent)) {
            throw new RuntimeException("主题不存在");
        }
        topicContent.setViews(topicContent.getViews() + 1);
        topicContentService.updateById(topicContent);

        OpenTopicContentVO openTopicContentVO = BeanUtil.copyProperties(topicContent, OpenTopicContentVO.class);
        openTopicContentVO.setAuthorName(websiteUsersService.getAuthorName(topicContent.getAuthor()));
        //分页查出楼层回复
        dto.setTopicId(topicId);
        Page<TopicRepliesVo> page = topicRepliesService.getFloorPageByOpen(dto);
        List<TopicRepliesVo> records = page.getRecords();
        records.forEach(topicRepliesVo -> {
            topicRepliesVo.setAuthorName(websiteUsersService.getAuthorName(topicRepliesVo.getAuthor()));
            dto.setFloorId(topicRepliesVo.getId());
            Page<TopicRepliesVo> floorPage = topicRepliesService.getPageByOpen(dto);
            topicRepliesVo.setFloorPage(floorPage);
        });
        openTopicContentVO.setRepliesPage(page);
        return openTopicContentVO;
    }


    private void validateReply(TopicRepliesDto topicRepliesDto, Long topicId) {
        if (StrUtil.isBlank(topicRepliesDto.getContent())) {
            throw new RuntimeException("回复不能为空");
        }
        if (StrUtil.isBlank(topicRepliesDto.getReplyType())) {
            throw new RuntimeException("回复类型不能为空");
        }
        if ("2".equals(topicRepliesDto.getReplyType()) && ObjectUtil.isNull(topicRepliesDto.getFloorId())) {
            throw new RuntimeException("回复对象不能为空");
        }
        if ("3".equals(topicRepliesDto.getReplyType()) && ObjectUtil.isNull(topicRepliesDto.getFloorId()) && ObjectUtil.isNull(topicRepliesDto.getReplyId())) {
            throw new RuntimeException("回复对象不能为空");
        }
        if ("2".equals(topicRepliesDto.getReplyType()) || "3".equals(topicRepliesDto.getReplyType())) {
            TopicReplies floorReply = topicRepliesService.getById(topicRepliesDto.getFloorId());
            if (ObjectUtil.isNull(floorReply)) {
                throw new RuntimeException("回复对象不存在");
            }
        }
        if ("3".equals(topicRepliesDto.getReplyType())) {
            TopicReplies replyTo = topicRepliesService.getById(topicRepliesDto.getReplyId());
            if (ObjectUtil.isNull(replyTo)) {
                throw new RuntimeException("回复对象不存在");
            }
            topicRepliesDto.setCiteAuthor(replyTo.getAuthor());
        }
        topicRepliesDto.setTopicId(topicId);
    }
}
